require "net/http"

module Gameworks
  class Notifier
    def configured?
      ![app_id, key, secret, base_url].any?(&:nil?)
    end

    def push(game, body)
      return unless configured?

      uri = URI(build_url(game))
      req = Net::HTTP::Post.new(uri)
      ssl = uri.scheme == 'https'

      req.basic_auth(key, secret)
      req.body = body
      req["Content-Type"] = "application/json"
      Net::HTTP.start(uri.hostname, uri.port, use_ssl: ssl) do |http|
        http.request(req)
      end
    end

    private

    def build_url(game)
      "%s/channel/%s/public/games/%s" % [base_url, app_id, game.id]
    end

    def app_id
      @app_id ||= ENV["PANDA_PUSH_APP_ID"]
    end

    def key
      @key ||= ENV["PANDA_PUSH_KEY"]
    end

    def secret
      @secret ||= ENV["PANDA_PUSH_SECRET"]
    end

    def base_url
      @base_url ||= ENV["PANDA_PUSH_URL"]
    end
  end
end
